import cv2


def video_info(video_path):
    cap = cv2.VideoCapture(video_path)
    w = cap.get(cv2.CAP_PROP_FRAME_WIDTH)
    h = cap.get(cv2.CAP_PROP_FRAME_HEIGHT)
    fps = cap.get(cv2.CAP_PROP_FPS)
    frame_count = cap.get(cv2.CAP_PROP_FRAME_COUNT)
    cap.release()
    return int(w), int(h), fps, int(frame_count)


def make_video_writer_from_video(video_path, output_video_path):
    w, h, fps, _ = video_info(video_path)
    fourcc = cv2.VideoWriter_fourcc(*'mp4v')
    return cv2.VideoWriter(output_video_path, fourcc, fps, (w, h))


if __name__ == '__main__':
    print(video_info(r"D:\Code\aigc-event-highlights\tennis\tennis.mkv"))



import cv2
import numpy as np
from skimage import transform as trans

arcface_dst = np.array(
    [[38.2946, 51.6963], [73.5318, 51.5014], [56.0252, 71.7366],
     [41.5493, 92.3655], [70.7299, 92.2041]],
    dtype=np.float32)


def clip_face(image, src_kps=None, yaw=0.0, face_size=112, top_offset=0, down_offset=0, left_offset=0, right_offset=0,
              strict_align=False):
    face_ratio = face_size / 112
    offset_x = left_offset
    if not strict_align:
        offset_x = offset_x + 30 * face_ratio * yaw
    dest_kps = np.array(
        [[38.2946 * face_ratio + offset_x, 51.6963 * face_ratio + top_offset],
         [73.5318 * face_ratio + offset_x, 51.5014 * face_ratio + top_offset],
         [56.0252 * face_ratio + offset_x, 71.7366 * face_ratio + top_offset],
         [41.5493 * face_ratio + offset_x, 92.3655 * face_ratio + top_offset],
         [70.7299 * face_ratio + offset_x, 92.2041 * face_ratio + top_offset]],
        dtype=np.float32)
    tform = trans.SimilarityTransform()
    tform.estimate(src_kps, dest_kps)
    M = tform.params[0:2, :]
    warped = cv2.warpAffine(image, M, (face_size + left_offset + right_offset, face_size + top_offset + down_offset),
                            borderValue=0.0)
    return warped, M



from datetime import datetime, timedelta

time_str_format = ['%Y-%m-%d %H:%M:%S.%f',
                   '%Y-%m-%d %H:%M:%S',
                   '%Y-%m-%d',
                   '%Y-%m-%dT%H:%M:%S.%f',
                   '%Y-%m-%dT%H:%M:%S']


def str2timestamp(time_str):
    try:
        return int(time_str)
    except:

        for str_f in time_str_format:
            try:
                date_time_obj = datetime.strptime(time_str, str_f)
                return int(date_time_obj.timestamp() * 1000)
            except:
                continue
    raise Exception('format time error')


def timestamp_ms2str(timestamp_ms):
    dt_object = datetime.fromtimestamp(timestamp_ms / 1000.0)

    # 格式化datetime对象为指定格式
    formatted_date = dt_object.strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]

    return formatted_date


def ms_to_str(milliseconds, split_char=':'):
    milliseconds = int(milliseconds)
    seconds, milliseconds = divmod(milliseconds, 1000)
    minutes, seconds = divmod(seconds, 60)
    hours, minutes = divmod(minutes, 60)

    return f"{hours:02d}{split_char}{minutes:02d}{split_char}{seconds:02d}.{milliseconds:03d}"


def ms_to_str_auto(milliseconds, include_split_char=True, include_ms=True):
    milliseconds = int(milliseconds)

    # 判断是否需要包含毫秒部分
    if not include_ms:
        milliseconds_part = ''
    else:
        if include_split_char:
            milliseconds_part = f".{milliseconds % 1000:03d}"
        else:
            milliseconds_part = f"{milliseconds % 1000:03d}"

    # 如果毫秒数大于等于1970年（63115200000）则视为完整时间戳
    if milliseconds >= 63115200000:
        end_date = datetime(1970, 1, 1) + timedelta(milliseconds=milliseconds)

        if include_split_char:
            formatted_time = end_date.strftime("%Y-%m-%d %H:%M:%S")
        else:
            formatted_time = end_date.strftime("%Y%m%d%H%M%S")

        return f"{formatted_time}{milliseconds_part}"

    else:
        # 否则表示时间段，只显示时分秒
        seconds, millis = divmod(milliseconds, 1000)
        minutes, seconds = divmod(seconds, 60)
        hours, minutes = divmod(minutes, 60)

        if include_split_char:
            time_str = f"{hours:02d}:{minutes:02d}:{seconds:02d}"
        else:
            time_str = f"{hours:02d}{minutes:02d}{seconds:02d}"

        return f"{time_str}{milliseconds_part}"


def ms_to_str_auto(milliseconds, include_split_char=True, include_ms=True):
    milliseconds = int(milliseconds)

    # 判断是否需要包含毫秒部分
    if not include_ms:
        milliseconds_part = ''
    else:
        if include_split_char:
            milliseconds_part = f".{milliseconds % 1000:03d}"
        else:
            milliseconds_part = f"{milliseconds % 1000:03d}"

    # 如果毫秒数大于等于1970年（63115200000）则视为完整时间戳
    if milliseconds >= 63115200000:
        end_date = datetime(1970, 1, 1) + timedelta(milliseconds=milliseconds)

        if include_split_char:
            formatted_time = end_date.strftime("%Y-%m-%d %H:%M:%S")
        else:
            formatted_time = end_date.strftime("%Y%m%d%H%M%S")

        return f"{formatted_time}{milliseconds_part}"

    else:
        # 否则表示时间段，只显示时分秒
        seconds, millis = divmod(milliseconds, 1000)
        minutes, seconds = divmod(seconds, 60)
        hours, minutes = divmod(minutes, 60)

        if include_split_char:
            time_str = f"{hours:02d}:{minutes:02d}:{seconds:02d}"
        else:
            time_str = f"{hours:02d}{minutes:02d}{seconds:02d}"

        return f"{time_str}{milliseconds_part}"


if __name__ == '__main__':
    # 示例调用
    print(ms_to_str_auto(123456789000, include_split_char=True, include_ms=True))  # 有分隔符，含毫秒
    print(ms_to_str_auto(123456789000, include_split_char=True, include_ms=False))  # 有分隔符，含毫秒
    print(ms_to_str_auto(123456789000, include_split_char=False, include_ms=False))  # 无分隔符，不含毫秒
    print(ms_to_str_auto(123456789000, include_split_char=False, include_ms=True))  # 无分隔符，不含毫秒
    print(ms_to_str_auto(5000, include_split_char=True, include_ms=True))  # 短时间，有分隔符，含毫秒
    print(ms_to_str_auto(5000, include_split_char=True, include_ms=False))  # 短时间，有分隔符，含毫秒
    print(ms_to_str_auto(5000, include_split_char=False, include_ms=False))  # 短时间，无分隔符，不含毫秒
    print(ms_to_str_auto(5000, include_split_char=False, include_ms=True))  # 短时间，无分隔符，不含毫秒



import os


def update_nested_dict(original, updates):
    """
    递归更新嵌套字典，存在的 key 替换，不存在的保留。
    :param original: 原始字典（会被修改）
    :param updates: 要更新的内容（新字典）
    :return: 更新后的原始字典
    """
    for key, value in updates.items():
        if isinstance(value, dict) and key in original and isinstance(original[key], dict):
            # 如果原字典中有该 key 且都是 dict，递归处理
            update_nested_dict(original[key], value)
        else:
            # 否则直接覆盖或新增
            original[key] = value
    return original


class EnhanceDict(dict):
    def __init__(self, data=None):
        # 初始化时将输入数据转换为 ConfigDict 类型
        super().__init__(data or {})

    def __getattr__(self, name):
        """
        通过属性访问键值。
        """
        value = self.get(name)
        if value is None:
            return NoneEnhance()
        if isinstance(value, dict):
            return EnhanceDict(value)
        return value

    def __setattr__(self, name, value):
        """
        通过属性设置键值。
        """
        self[name] = value

    def __getitem__(self, key):
        """
        重写 __getitem__ 方法，支持嵌套字典返回 ConfigDict。
        """
        value = super().__getitem__(key)
        if isinstance(value, dict):
            return EnhanceDict(value)
        return value

    def __setitem__(self, key, value):
        """
        重写 __setitem__ 方法，自动将字典类型的值转换为 ConfigDict。
        """
        if isinstance(value, dict):
            value = EnhanceDict(value)
        super().__setitem__(key, value)

    def update_from(self, other):
        self.update(update_nested_dict(dict(self), dict(other)))

    def get(self, key, default=None):
        """
        重写 get 方法，支持嵌套字典返回 ConfigDict。
        """
        value = super().get(key, default)
        if isinstance(value, dict):
            return EnhanceDict(value)
        return value


class NoneEnhance:
    def __getattr__(self, name):
        return NoneEnhance()

    def __bool__(self):
        return False


def value_or_default(value, default=None):
    if isinstance(value, NoneEnhance):
        return default
    return value


def read_prefixed_env_vars(prefix):
    """
    读取以指定前缀开头的环境变量，并将其转换为小写、嵌套的字典。

    :param prefix: 环境变量前缀（例如 "APP_"）
    :return: 转换后的嵌套字典
    """
    # 确保前缀是大写的，避免大小写问题
    prefix = prefix.upper()
    result = {}

    for key, value in os.environ.items():
        if key.startswith(prefix):
            # 去掉前缀部分并转为小写
            stripped_key = key[len(prefix):].lower()
            # 使用双下划线分割各个部分
            parts = stripped_key.split('__')

            current_level = result
            for part in parts[:-1]:
                if part not in current_level:
                    current_level[part] = {}
                current_level = current_level[part]

            # 尝试将字符串值解析为Python数据类型
            try:
                namespace = {}
                exec(f'data={value}', namespace)
                data = namespace['data']
            except:
                data = value  # 如果解析失败则保留原始字符串

            # 设置最终值
            current_level[parts[-1]] = data

            # 如果parts长度为1，则直接设置在result上
            if len(parts) == 1:
                result[stripped_key] = data

    return result


if __name__ == '__main__':
    import os

    data = EnhanceDict({"a": {"b": 1,"d":23},"f":3})
    print(data)
    data.update_from({"a": {"b": 2,"d":32323},"e":12})
    print(data)



import os
import cv2
import random
import argparse
from threading import Thread
from queue import Queue
from tqdm import tqdm


def try_open_video(url, timeout=10):
    cap = cv2.VideoCapture(url)
    result_queue = Queue()

    def target():
        if cap.isOpened():
            total_frames = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
            result_queue.put((cap, total_frames))
        else:
            result_queue.put((None, 0))

    thread = Thread(target=target)
    thread.start()
    thread.join(timeout)

    if thread.is_alive():
        cap.release()
        return None, 0
    else:
        return result_queue.get()


def main():
    parser = argparse.ArgumentParser(description='Extract frames from videos')
    parser.add_argument('--input', '-i', type=str, default='video_list.txt',
                        help='Input file containing video URLs')
    parser.add_argument('--output', '-o', type=str, default='output_images',
                        help='Output directory for extracted frames')
    parser.add_argument('--frames', '-f', type=int, default=10,
                        help='Number of frames to extract per video')
    args = parser.parse_args()

    video_list_file = args.input
    output_root = args.output
    max_frames = args.frames

    with open(video_list_file, 'r') as f:
        video_urls = [line.strip() for line in f if line.strip()]

    for url in tqdm(video_urls, desc="Processing videos"):
        try:
            video_filename = os.path.basename(url)
            video_name = os.path.splitext(video_filename)[0]
            output_dir = os.path.join(output_root, video_name)

            os.makedirs(output_dir, exist_ok=True)

            existing_images = [f for f in os.listdir(output_dir)
                               if f.lower().endswith(('.jpg', '.jpeg', '.png'))]
            current_count = len(existing_images)

            if current_count >= max_frames:
                tqdm.write(f"Skipping {video_name}, already has {current_count} frames.")
                continue

            needed = max_frames - current_count

            selected_file = os.path.join(output_dir, 'selected_frames.txt')
            selected_frames = []
            if os.path.exists(selected_file):
                with open(selected_file, 'r') as f:
                    selected_frames = [int(line.strip()) for line in f]
            else:
                with open(selected_file, 'w') as f:
                    pass

            cap, total_frames = try_open_video(url, timeout=10)
            if not cap or total_frames <= 0:
                tqdm.write(f"Failed to open video {url} within timeout or no frames available")
                continue

            available_frames = list(set(range(total_frames)) - set(selected_frames))
            if len(available_frames) < needed:
                needed = len(available_frames)
                if needed <= 0:
                    cap.release()
                    continue

            new_selected = random.sample(available_frames, needed)
            selected_frames.extend(new_selected)

            with open(selected_file, 'a') as f:
                for frame in new_selected:
                    f.write(f"{frame}\n")

            for frame_num in new_selected:
                cap.set(cv2.CAP_PROP_POS_FRAMES, frame_num)
                ret, frame = cap.read()
                if not ret:
                    tqdm.write(f"Failed to read frame {frame_num} from {url}")
                    continue
                img_path = os.path.join(output_dir, f"{video_name}_{frame_num}.jpg")
                cv2.imwrite(img_path, frame)

            cap.release()

        except Exception as e:
            tqdm.write(f"Error processing {url}: {str(e)}")
            continue


if __name__ == "__main__":
    main()



import json
import math
import time
from abc import abstractmethod
import logging
import cv2
import os
import numpy

def_logger = logging.getLogger(__name__)
video_keywords = ('.mp4', '.avi', '.mkv', '.mov')

try:
    import torch
    import torch.nn.functional as F

except ImportError:
    print('PyTorch not found.')


def tensor2numpy_bgr(img_tensor):
    img = (img_tensor * 255.0).cpu().numpy()
    img = img.astype(numpy.uint8)
    if len(img.shape) == 3 and img.shape[0] == 3:  # 对于RGB图像
        img = numpy.transpose(img, (1, 2, 0))  # 更改维度顺序
        img = cv2.cvtColor(img, cv2.COLOR_RGB2BGR)  # OpenCV使用BGR格式
        return img


class StreamDecoder:
    def __init__(self, video_path, start=0, end=None, max_fps=500, out_path=None, logger=def_logger,
                 tqdm_enable=True, control_type='frame', is_live=None, frame_type='numpy_rgb', **kwargs):
        # numpy_rgb,numpy_bgr,pillow,tensor
        self.frame_type = frame_type or 'numpy_rgb'

        if logger is None:
            self.logger = def_logger
        else:
            self.logger = logger
        self.video_path = video_path

        if is_live is None:
            if video_path.startswith('rtmp') or video_path.startswith('rtsp'):
                self.is_live = True
            elif video_path.endswith(video_keywords):
                self.is_live = False
            else:
                raise ValueError('live is null')
        else:
            self.is_live = is_live
        if self.is_live and start > 0:
            raise Exception('start must be 0 for live stream')
        cap = cv2.VideoCapture(video_path)
        try:
            if cap.isOpened() is False:
                raise Exception(f"无法打开视频: {video_path}")
            self.width = int(cap.get(cv2.CAP_PROP_FRAME_WIDTH))
            self.height = int(cap.get(cv2.CAP_PROP_FRAME_HEIGHT))
            self.fps = cap.get(cv2.CAP_PROP_FPS)
            frame_count = int(cap.get(cv2.CAP_PROP_FRAME_COUNT))
            if abs(self.fps - 25) < 0.2:
                self.fps = 25.
            elif abs(self.fps - 30) < 0.2:
                self.fps = 30.

            # self.ori_start = start
            # self.ori_end = end
            self.ori_max_fps = max_fps or self.fps
            self.target_fps = min(self.ori_max_fps, self.fps)
            self.total_frame = None
            if not self.is_live and control_type == 'frame':
                self.start_frame = start
                self.start_time = start / self.fps * 1000
                self.end_frame = end if end is not None and end <= frame_count else frame_count
                self.end_time = self.end_frame / self.fps * 1000
                self.total_frame = self.end_frame - self.start_frame
            elif not self.is_live and control_type == 'time':
                self.start_frame = int(start / 1000 * self.fps)
                self.start_time = start
                self.end_frame = int(end / 1000 * self.fps) if end is not None else frame_count
                self.end_frame = min(self.end_frame, frame_count)
                self.end_time = self.end_frame / self.fps * 1000
                self.total_frame = self.end_frame - self.start_frame
            elif self.is_live and control_type == 'frame':
                # self.start_frame = start
                self.start_frame = 0
                self.start_time = 0
                self.end_frame = end
                if end is not None:
                    self.end_time = end / self.fps * 1000
                    self.total_frame = self.end_frame - self.start_frame
            elif self.is_live and control_type == 'time':
                self.start_frame = int(start / 1000 * self.fps)
                self.start_time = start
                self.start_frame = 0
                self.end_frame = None if end is None else int(end / 1000 * self.fps)
                if end is not None:
                    self.end_time = end
                    self.total_frame = self.end_frame - self.start_frame

        finally:
            cap.release()

        if tqdm_enable and self.total_frame is not None:
            from tqdm import tqdm
            self.tqdm_c = tqdm(total=self.__len__(), unit='frame', desc=f'{os.path.basename(video_path)}')
        else:
            self.tqdm_c = None

        self.out_writer = self.__make_video_writer__(out_path) if out_path is not None else None

        self.kwargs = kwargs

        self.stop_flag = False

        self.fram_iter = self.decode_iter()

        self.tqdm_count = 0

    @abstractmethod
    def decode_iter(self):
        pass

    def __make_video_writer__(self, output_video_path):
        fourcc = cv2.VideoWriter_fourcc(*'mp4v')
        return cv2.VideoWriter(output_video_path, fourcc, self.fps, (self.width, self.height))

    def __enter__(self):
        if self.out_writer is not None:
            return self, self.out_writer
        else:
            return self, None

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.stop()

    def stop(self):
        try:
            # 尝试发送停止信号，适用于协程或生成器
            self.stop_flag = True
            if self.out_writer is not None:
                self.out_writer.release()
        except Exception:
            pass

    def __next__(self):
        data = next(self.fram_iter)
        if self.tqdm_c is not None:
            # update_data = data['src_frame_idx'] - self.tqdm_count
            self.tqdm_c.update(1)
            # self.tqdm_count = data['src_frame_idx']
        return data

    def __iter__(self):
        return self

    def __len__(self):
        if self.total_frame:
            return math.ceil(self.total_frame / self.fps * self.target_fps)
        else:
            return 0


class SDKStreamDecoder(StreamDecoder):
    def __init__(self, video_path, start=0, end=None, max_fps=500, out_path=None, logger=def_logger, tqdm_enable=True,
                 control_type='frame', is_live=None, frame_type='numpy_rgb', **kwargs):

        super().__init__(video_path, start, end, max_fps, out_path, logger, tqdm_enable, control_type, is_live,
                         frame_type, **kwargs)

    def decode_iter(self):
        if self.stop_flag:
            return
        import xvdecoder
        sei_key = self.kwargs.get('sei_key', None)
        ret = xvdecoder.start_decoder(self.video_path, sei_key or '', 10, "xvdecoder.log")

        ori_frame_count = 0
        target_frame_count = 0
        seek_idx = 0
        try:
            while True:
                if self.stop_flag:
                    break

                # 如果帧数够了停止
                if self.total_frame is not None and ori_frame_count >= self.total_frame:
                    break
                tensor = xvdecoder.decode_frame_to_tensor()
                if tensor is None or tensor.numel() == 0:
                    # 检查内部c++任务是否已经结束
                    is_finished = xvdecoder.check_finished()

                    if is_finished > 0:
                        print("is_finished ", is_finished)
                        break
                    else:
                        time.sleep(0.1)
                        continue
                if seek_idx < self.start_frame:
                    seek_idx += 1
                    continue

                next_time_point = target_frame_count / self.target_fps * 1000
                current_time_point = int(ori_frame_count / self.fps * 1000)
                if current_time_point >= next_time_point:
                    image_tensor = tensor.float() / 255.0
                    image_tensor = image_tensor.permute(2, 0, 1)
                    image_tensor = image_tensor[[2, 1, 0], :, :]
                    sei_msg = xvdecoder.get_curr_sei_msg()
                    data = {
                        'src_fps': self.fps,
                        'fps': self.target_fps,
                        'offset_idx': self.start_frame,
                        'offset_time': self.start_time,
                        'src_frame_idx': ori_frame_count,
                        'src_frame_time': round(current_time_point),
                        'target_frame_idx': target_frame_count,
                        'frame': image_tensor,
                        # 'target_frame_time': target_frame_count * 1000 / target_fps,
                        'sei': sei_msg,
                    }
                    yield data

                    target_frame_count += 1
                ori_frame_count += 1

        except:
            self.logger.exception("xvdecoder error")
        finally:
            xvdecoder.stop_decoder()


class AVStreamDecoder(StreamDecoder):
    def __init__(self, video_path, start=0, end=None, max_fps=500, out_path=None, logger=def_logger, tqdm_enable=True,
                 control_type='frame', is_live=None, to_device='cpu', frame_type='numpy_rgb', gpu_decoder=False,
                 **kwargs):
        self.to_device = to_device
        self.gpu_decoder = gpu_decoder
        super().__init__(video_path, start, end, max_fps, out_path, logger, tqdm_enable, control_type, is_live,
                         frame_type, **kwargs)

    def yuv420p_to_rgb_gpu(self, y, u, v):
        y = torch.tensor(y, dtype=torch.float32, device=self.to_device)
        u = torch.tensor(u, dtype=torch.float32, device=self.to_device)
        v = torch.tensor(v, dtype=torch.float32, device=self.to_device)

        # 上采样 U 和 V 到 Y 的分辨率
        u_upsampled = u.repeat_interleave(2, dim=0).repeat_interleave(2, dim=1)
        v_upsampled = v.repeat_interleave(2, dim=0).repeat_interleave(2, dim=1)

        # YUV 转 RGB 公式（BT.601）
        r = y + 1.402 * (v_upsampled - 128)
        g = y - 0.34414 * (u_upsampled - 128) - 0.71414 * (v_upsampled - 128)
        b = y + 1.772 * (u_upsampled - 128)

        # 合并通道并裁剪到 [0, 255]
        rgb = torch.stack([r, g, b], dim=0).clamp(0, 255)

        # 归一化到 [0, 1]
        rgb_normalized = rgb / 255.0

        return rgb_normalized  # 返回 float32 类型，范围 [0, 1]

    def decode_iter(self):
        if self.stop_flag:
            return
        import av

        if self.kwargs.get('ffmpeg_path', None):
            ffmpeg_path = self.kwargs.get('ffmpeg_path', None)

            # if 'LD_LIBRARY_PATH' in os.environ:
            #     os.environ['LD_LIBRARY_PATH'] = f"{ffmpeg_path}/lib:" + os.environ['LD_LIBRARY_PATH']
            # else:
            #     os.environ['LD_LIBRARY_PATH'] = f"{ffmpeg_path}/lib"
            # if 'PATH' in os.environ:
            #     os.environ['PATH'] = f"{ffmpeg_path}/bin:" + os.environ['PATH']
            # else:
            #     os.environ['PATH'] = f"{ffmpeg_path}/bin"
            try:
                av.utils.ffmpeg_path = ffmpeg_path
            except:
                self.logger.exception('ffmpeg path set error')

        sei_key = self.kwargs.get('sei_key', None)

        container = None
        if self.gpu_decoder:
            try:
                from av.codec.hwaccel import HWAccel, hwdevices_available
                if 'cuda' in hwdevices_available():
                    hwaccel = HWAccel(device_type='cuda', allow_software_fallback=False)
                    container = av.open(self.video_path, hwaccel=hwaccel, timeout=5)
            except:
                self.logger.exception('gpu ffmpeg error')

        if container is None:
            container = av.open(self.video_path, timeout=5)
        if self.stop_flag:
            return
        with container:
            try:
                is_hwaccel = container.streams.video[0].codec_context.is_hwaccel
            except:
                is_hwaccel = False
            stream = next(s for s in container.streams if s.type == 'video')
            total_frames = int(stream.frames)
            # fps = float(stream.average_rate)
            time_base = stream.time_base
            color_format = stream.format.name
            self.logger.info(
                f'video info --> video:{self.video_path}, color_format:{color_format}, '
                f'total_frames:{total_frames}, fps:{self.fps}, is_hwaccel:{is_hwaccel}')

            if self.start_time is not None and self.start_time > 0:
                container.seek(
                    int(self.start_time / 1000 / time_base),
                    # any_frame=True,
                    # backward=False,
                    stream=stream
                )

            target_fps = min(self.ori_max_fps, self.fps)

            if self.frame_type == "tensor":
                from torchvision import transforms
                to_tensor = transforms.ToTensor()

            # 需要seek
            seek_tag = self.start_time is not None and self.start_time > 0

            ori_frame_count = 0
            target_frame_count = 0
            for frame in container.decode(stream):
                # 遇到停止标识停止
                if self.stop_flag:
                    break

                # 由于seek只是到关键帧需要继续到指定位置
                if seek_tag and float(frame.time * 1000) < self.start_time:
                    continue
                seek_tag = False

                # 如果帧数够了停止
                if self.total_frame is not None and ori_frame_count >= self.total_frame:
                    break
                next_time_point = round(target_frame_count / self.target_fps * 1000)
                current_time_point = round(ori_frame_count / self.fps * 1000)

                if current_time_point >= next_time_point:
                    sei_message_str = None
                    if sei_key is not None:
                        try:
                            for sd in frame.side_data:
                                sd_str = bytes(sd).decode('utf-8')
                                if sd_str.startswith(sei_key):
                                    sei_message_str = sd_str[len(sei_key):]
                                    break
                        except:
                            self.logger.error('解析sei信息错误')
                    if frame.format.name == 'nv12' and self.gpu_decoder and self.frame_type == 'tensor':
                        yuv_data = frame.reformat(format='yuv420p')
                        y_plane = yuv_data.planes[0]
                        y_plane_n = numpy.frombuffer(y_plane, dtype=numpy.uint8).reshape(
                            (y_plane.height, y_plane.width))
                        u_plane = yuv_data.planes[1]
                        u_plane_n = numpy.frombuffer(u_plane, dtype=numpy.uint8).reshape(
                            (u_plane.height, u_plane.width))
                        v_plane = yuv_data.planes[2]
                        v_plane_n = numpy.frombuffer(v_plane, dtype=numpy.uint8).reshape(
                            (v_plane.height, v_plane.width))

                        final_frame = self.yuv420p_to_rgb_gpu(y_plane_n, u_plane_n, v_plane_n)
                    else:
                        if self.frame_type == 'pillow':
                            final_frame = frame.to_image()
                        elif self.frame_type == 'numpy_rgb':
                            final_frame = frame.to_ndarray(format='rgb24')
                        elif self.frame_type == 'numpy_bgr':
                            final_frame = frame.to_ndarray(format='bgr24')
                        elif self.frame_type == 'tensor':
                            if self.to_device.startswith('cuda') and frame.format.name == 'yuv420p':
                                yuv_data = frame.reformat(format='yuv420p')
                                y_plane = yuv_data.planes[0]
                                y_plane_n = numpy.frombuffer(y_plane, dtype=numpy.uint8).reshape(
                                    (y_plane.height, y_plane.width))
                                u_plane = yuv_data.planes[1]
                                u_plane_n = numpy.frombuffer(u_plane, dtype=numpy.uint8).reshape(
                                    (u_plane.height, u_plane.width))
                                v_plane = yuv_data.planes[2]
                                v_plane_n = numpy.frombuffer(v_plane, dtype=numpy.uint8).reshape(
                                    (v_plane.height, v_plane.width))

                                final_frame = self.yuv420p_to_rgb_gpu(y_plane_n, u_plane_n, v_plane_n)
                            else:
                                final_frame = to_tensor(frame.to_ndarray(format='rgb24'))
                    data = {
                        'src_fps': self.fps,
                        'fps': target_fps,
                        'offset_idx': self.start_frame,
                        'offset_time': self.start_time,
                        'src_frame_idx': ori_frame_count,
                        'src_frame_time': round(current_time_point),
                        'target_frame_idx': target_frame_count,
                        'frame': final_frame,
                        # 'target_frame_time': target_frame_count * 1000 / target_fps,
                        'sei': sei_message_str,
                    }
                    yield data
                    # if self.stop_flag or (self.end_frame is not None and self.end_frame <= ori_frame_count):
                    #     return
                    target_frame_count += 1
                ori_frame_count += 1


class CVStreamDecoder(StreamDecoder):
    def __init__(self, video_path, start=0, end=None, max_fps=500, out_path=None, logger=def_logger, tqdm_enable=True,
                 control_type='frame', is_live=None, to_device='cpu', frame_type='numpy_rgb', **kwargs):
        self.to_device = to_device
        super().__init__(video_path, start, end, max_fps, out_path, logger, tqdm_enable, control_type, is_live,
                         frame_type, **kwargs)

    def decode_iter(self):
        from PIL import Image
        if self.stop_flag:
            return

        # 使用 OpenCV 的 VideoCapture
        cap = cv2.VideoCapture(self.video_path)
        try:
            if not cap.isOpened():
                self.logger.error("无法打开视频文件")
                return

            self.logger.info(
                f'video info --> video:{self.video_path}, total_frames:{self.total_frame}, fps:{self.fps}, is_hwaccel:False'
            )

            # 设置起始时间跳转
            if self.start_time is not None and self.start_time > 0:
                cap.set(cv2.CAP_PROP_POS_MSEC, self.start_time)

            ori_frame_count = 0
            target_frame_count = 0

            # 转换为 tensor 的 transform
            if self.frame_type == "tensor":
                from torchvision import transforms
                to_tensor = transforms.ToTensor()
            else:
                pass  # 其他格式处理

            # 逐帧读取
            while True:
                # 遇到停止标识停止
                if self.stop_flag:
                    break

                # 如果帧数够了停止
                if self.total_frame is not None and ori_frame_count >= self.total_frame:
                    break

                ret, frame_bgr = cap.read()
                if not ret:
                    break

                next_time_point = round(target_frame_count / self.target_fps * 1000)
                current_time_point = round(ori_frame_count / self.fps * 1000)

                if current_time_point >= next_time_point:
                    sei_message_str = None  # OpenCV 不支持提取 SEI 消息

                    # 处理帧格式转换
                    if self.frame_type == 'pillow':
                        final_frame = Image.fromarray(cv2.cvtColor(frame_bgr, cv2.COLOR_BGR2RGB))
                    elif self.frame_type == 'numpy_rgb':
                        final_frame = cv2.cvtColor(frame_bgr, cv2.COLOR_BGR2RGB)
                    elif self.frame_type == 'numpy_bgr':
                        final_frame = frame_bgr
                    elif self.frame_type == 'tensor':
                        # OpenCV 帧为 BGR HWC，转换为 RGB CHW
                        frame_rgb = cv2.cvtColor(frame_bgr, cv2.COLOR_BGR2RGB)
                        final_frame = to_tensor(frame_rgb).to(self.to_device)
                    else:
                        final_frame = None  # 默认返回 numpy_bgr

                    # 构造返回数据字典
                    data = {
                        'src_fps': self.fps,
                        'fps': self.target_fps,
                        'offset_idx': self.start_frame,
                        'offset_time': self.start_time,
                        'src_frame_idx': ori_frame_count,
                        'src_frame_time': round(current_time_point),
                        'target_frame_idx': target_frame_count,
                        'frame': final_frame,
                        'sei': sei_message_str,
                    }
                    yield data

                    target_frame_count += 1
                ori_frame_count += 1
        finally:
            cap.release()  # 清理资源


class MockStreamDecoder(StreamDecoder):
    def __init__(self, video_path, start=0, end=None, max_fps=500, out_path=None, logger=def_logger, tqdm_enable=True,
                 control_type='frame', is_live=None, to_device='cpu', frame_type='numpy_rgb', **kwargs):
        self.to_device = to_device
        super().__init__(video_path, start, end, max_fps, out_path, logger, tqdm_enable, control_type, is_live,
                         frame_type, **kwargs)

    def decode_iter(self):
        # ori_frame_count = 0
        target_frame_count = 0
        for ori_frame_count in range(self.total_frame):
            # 遇到停止标识停止
            if self.stop_flag:
                break
            # 如果帧数够了停止
            if self.total_frame is not None and ori_frame_count >= self.total_frame:
                break
            next_time_point = round(target_frame_count / self.target_fps * 1000)
            current_time_point = round(ori_frame_count / self.fps * 1000)
            if current_time_point >= next_time_point:
                data = {
                    'weight': self.width,
                    'height': self.height,
                    'src_fps': self.fps,
                    'fps': self.target_fps,
                    'offset_idx': self.start_frame,
                    'offset_time': self.start_time,
                    'src_frame_idx': ori_frame_count,
                    'src_frame_time': round(ori_frame_count / self.target_fps * 1000),
                    'target_frame_idx': target_frame_count,
                    'frame': None,
                    'sei': json.dumps({'utc': round(ori_frame_count / self.target_fps * 1000)}),
                }
                yield data
                target_frame_count += 1


if __name__ == '__main__':
    from aabd.base.log_setting import get_set_once_logger
    from aabd.base.path_util import to_absolute_path_str
    import os

    os.environ['PROJECT_ROOT'] = os.path.abspath("../")
    logger = get_set_once_logger(log_type=['console'])
    with CVStreamDecoder(to_absolute_path_str('files/111.mp4'), start=1000, end=3000, max_fps=30, logger=logger,frame_type="numpy_bgr",control_type='time') as (
            decoder,
            _):
        for frame in decoder:
            cv2.imwrite(f'images/{frame["src_frame_idx"]}.png', frame['frame'])
            frame['frame'] = None
            logger.info(frame)



import logging
import threading
import time

from .channel import MqReceiver, MqSender
import pika
from pika.exchange_type import ExchangeType

logger = logging.getLogger('rabbitmq')


def get_connection_from_config(host, port, username, password, virtual_host, heartbeat=None):
    # 连接服务器
    credentials = pika.PlainCredentials(username=username, password=password) if username and password else None
    connection = pika.BlockingConnection(
        # heartbeat=0会导致重启的时候老的consumer channel不会被server端主动检测到超时而断开
        (
            pika.ConnectionParameters(host=host, port=port, credentials=credentials, virtual_host=virtual_host,
                                      heartbeat=heartbeat)
            if heartbeat is not None else
            pika.ConnectionParameters(host=host, port=port, credentials=credentials, virtual_host=virtual_host)
        )
        if credentials else
        (
            pika.ConnectionParameters(host=host, port=port, virtual_host=virtual_host, heartbeat=heartbeat)
            if heartbeat is not None else
            pika.ConnectionParameters(host=host, port=port, virtual_host=virtual_host)
        )
    )
    return connection


class RabbitmqReceiver(MqReceiver):

    def __init__(self, host="127.0.0.1", port=5672, username='admin', password='admin', virtual_host='/',
                 heartbeat=None,
                 consumer_exchange=None, consumer_routing_key=None, consumer_queue=None,
                 exchange_type=ExchangeType.topic, configuration=None):
        super().__init__()
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.virtual_host = virtual_host
        self.heartbeat = heartbeat
        if configuration is None:
            configuration = {}
        self.consumer_queue = consumer_queue
        self.consumer_routing_key = consumer_routing_key
        self.consumer_exchange = consumer_exchange
        self.exchange_type = exchange_type
        self.extra_configuration = configuration.get('extra') if configuration.get('extra') else {}
        self._link()

    def _link(self):
        consumer_connection: pika.BlockingConnection = get_connection_from_config(self.host, self.port, self.username,
                                                                                  self.password,
                                                                                  self.virtual_host, self.heartbeat)
        logger.info(f'consumer_connection_dict = {consumer_connection.__dict__}')

        self.consumer_message_channel: pika.adapters.blocking_connection.BlockingChannel = consumer_connection.channel()
        logger.info(f'consumer_channel_dict = {self.consumer_message_channel.__dict__}')
        logger.info(f'consumer_tag = {self.consumer_message_channel.consumer_tags}')

        if self.consumer_exchange is not None:
            self.consumer_message_channel.exchange_declare(exchange=self.consumer_exchange,
                                                           exchange_type=self.exchange_type,
                                                           durable=True)
        if self.consumer_queue is not None and self.consumer_routing_key is not None:
            self.consumer_message_channel.queue_declare(queue=self.consumer_queue, durable=True)
            self.consumer_message_channel.queue_bind(queue=self.consumer_queue, exchange=self.consumer_exchange,
                                                     routing_key=self.consumer_routing_key)

    def __on_message_callback(self, channel, method, properties, message):
        logger.debug(f'message (type is {type(message)}) received through Queue {self.consumer_queue}: {message}')

        thread = threading.Thread(target=super()._receive, args=[message],
                                  kwargs={'method': method, "properties": properties})
        thread.start()

        while thread.is_alive():  # Loop while the thread is processing
            logger.debug(f' loop {self.consumer_queue} process_data_events while the thread is processing message')
            channel._connection.sleep(1.0)
        logger.debug(
            f'Back from thread for Queue {self.consumer_queue}, and channel has {channel.get_waiting_message_count()} messages more')
        if self.auto_ack is False:
            logger.info(f'basic_ack: {method.delivery_tag}')
            channel.basic_ack(delivery_tag=method.delivery_tag)

    def start_receiving(self):
        super().start_receiving()
        # 新建 thread 启动
        threading.Thread(target=self.reconnected_logic).start()

    def reconnected_logic(self):
        thread = threading.Thread(target=self._start_consuming)
        thread.start()
        while True:
            a = self.consumer_message_channel.is_open
            b = self.consumer_message_channel.is_closed
            c = self.consumer_message_channel.connection.is_open
            d = self.consumer_message_channel.connection.is_closed
            if a == b or c == d or a != c:
                logger.info(f'WEIRED! {self.consumer_queue} consumer channel status: {a} {b} {c} {d}')
            if not b:
                if not thread.is_alive():
                    logger.info(f'AGAIN! {self.consumer_queue} consumer thread is dead')
                    thread = threading.Thread(target=self._start_consuming)
                    thread.start()
            else:
                logger.info(f'AGAIN! {self.consumer_queue} consumer channel is closed')
                if thread.is_alive():
                    logger.info(
                        f'{self.consumer_queue} consumer channel is closed, but blocked in consuming thread??? do stopping')
                    self.consumer_message_channel.stop_consuming()
                self._link()
                thread = threading.Thread(target=self._start_consuming)
                thread.start()
            time.sleep(2)

    def _start_consuming(self):
        # https://blog.csdn.net/m0_46825740/article/details/120220791
        try:
            self.consumer_message_channel.basic_qos(prefetch_count=1)
            self.auto_ack = self.extra_configuration.get('autoAck')
            if self.auto_ack is False:
                self.consumer_message_channel.basic_consume(queue=self.consumer_queue,
                                                            on_message_callback=self.__on_message_callback)
            else:
                self.auto_ack = True
                self.consumer_message_channel.basic_consume(queue=self.consumer_queue,
                                                            on_message_callback=self.__on_message_callback,
                                                            auto_ack=True)
            self.consumer_message_channel.start_consuming()
        except:
            logger.error(f'{self.consumer_queue} channel consuming error', exc_info=True, stack_info=True)
        finally:
            logger.info(f'{self.consumer_queue} consuming seems to have been stopped')


class RabbitmqSender(MqSender):

    def __init__(self, host="127.0.0.1", port=5672, username='admin', password='admin', virtual_host='/',
                 heartbeat=None, producer_exchange=None, producer_routing_key=None, exchange_type=ExchangeType.topic):
        super().__init__()
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.virtual_host = virtual_host
        self.heartbeat = heartbeat
        self.producer_exchange = producer_exchange
        self.exchange_type = exchange_type
        self.producer_routing_key = producer_routing_key
        self.producer_message_channel = None
        self.proactive_invoke_stop_event = None
        self._link()

    def _link(self):
        if self.producer_message_channel is not None:
            if not self.producer_message_channel.is_closed:
                self.producer_message_channel.close()
                logger.info(
                    f'close channel for producer working on {self.producer_exchange} + {self.producer_routing_key}')
            else:
                logger.info(
                    f'channel for producer working on {self.producer_exchange} + {self.producer_routing_key} has already been closed')
            # self.proactive_invoke_stop_event.set()  # 停止线程

        producer_connection = get_connection_from_config(self.host, self.port, self.username, self.password,
                                                         self.virtual_host, self.heartbeat)
        logger.info(
            f'producer_connection_dict = {producer_connection.__dict__}')
        self.producer_message_channel: pika.adapters.blocking_connection.BlockingChannel = producer_connection.channel()
        logger.info(
            f'producer_channel_dict = {self.producer_message_channel.__dict__}')
        if self.producer_exchange is not None:
            self.producer_message_channel.exchange_declare(exchange=self.producer_exchange,
                                                           exchange_type=self.exchange_type,
                                                           durable=True)

        # # 创建一个Event对象
        # self.proactive_invoke_stop_event: threading.Event = threading.Event()
        # threading.Thread(target=self.proactive_invoke_heartbeat, args=[self.proactive_invoke_stop_event]).start()

    def proactive_invoke_heartbeat(self, stop_event: threading.Event):
        while not stop_event.is_set():
            if self.producer_message_channel is not None and self.producer_message_channel.is_open:
                logger.info(
                    f' proactive_invoke loop {self.producer_exchange} + {self.producer_routing_key} process_data_events')
                self.producer_message_channel._connection.sleep(1.0)

    def sending(self, feedback, exchange=None, routing_key=None):
        super().sending(feedback=feedback)
        exchange = self.producer_exchange if exchange is None else exchange
        routing_key = self.producer_routing_key if routing_key is None else routing_key
        self._produce(feedback, exchange=exchange, routing_key=routing_key)

    def _produce(self, message, exchange=None, routing_key=None):
        logger.debug(
            f'message (type is {type(message)}) sent through Exchange {exchange} and RoutingKey {routing_key}: {message}')
        try:
            self.producer_message_channel.basic_publish(exchange=exchange,
                                                        routing_key=routing_key, body=message)
            logger.debug(
                f'message sent successfully through Exchange {exchange} and RoutingKey {routing_key}')
        except:
            logger.exception('消息发送异常，即将重试')
            self._link()
            try:
                self.producer_message_channel.basic_publish(exchange=exchange,
                                                            routing_key=routing_key, body=message)
                logger.debug(
                    f'message sent successfully through Exchange {exchange} and RoutingKey {routing_key}')
            except:
                logger.exception('消息发送异常，不重试')



import torch
from torch import nn
from torch.utils.checkpoint import checkpoint


class eDifFIQA(torch.nn.Module):
    """ eDifFIQA model consisting of a pretrained FR backbone (CosFace in the original implementation) and a quality regression MLP head.

    Args:
        base_model (torch.nn.Module): FR backbone used for feature extraction.
        mlp (torch.nn.Module): MLP used as a quality regression head.
        return_feat (bool): Flag for returning features, if set to True the model returns (features, qualities) otherwise only the qualities.
    """

    def __init__(self,
                 backbone_model: torch.nn.Module,
                 quality_head: torch.nn.Module,
                 return_feat: bool = True):
        super().__init__()

        self.base_model = backbone_model
        self.mlp = quality_head
        self.return_feat = return_feat

    def forward(self, x):
        feat = self.base_model(x)
        pred = self.mlp(feat)
        if self.return_feat:
            return (feat, pred)
        return pred


class MLP(torch.nn.Module):

    def __init__(self,
                 in_dim: int = 512,
                 hidden_dim: int = 1024,
                 out_dim: int = 1):
        super().__init__()

        self.l1 = torch.nn.Linear(in_dim, hidden_dim)
        self.ac = torch.nn.GELU()
        self.l2 = torch.nn.Linear(hidden_dim, out_dim)

    def forward(self, x):
        x = self.ac(self.l1(x))
        return self.l2(x)


using_ckpt = False


def conv3x3(in_planes, out_planes, stride=1, groups=1, dilation=1):
    """3x3 convolution with padding"""
    return nn.Conv2d(in_planes,
                     out_planes,
                     kernel_size=3,
                     stride=stride,
                     padding=dilation,
                     groups=groups,
                     bias=False,
                     dilation=dilation)


def conv1x1(in_planes, out_planes, stride=1):
    """1x1 convolution"""
    return nn.Conv2d(in_planes,
                     out_planes,
                     kernel_size=1,
                     stride=stride,
                     bias=False)


class IBasicBlock(nn.Module):
    expansion = 1

    def __init__(self, inplanes, planes, stride=1, downsample=None,
                 groups=1, base_width=64, dilation=1):
        super(IBasicBlock, self).__init__()
        if groups != 1 or base_width != 64:
            raise ValueError('BasicBlock only supports groups=1 and base_width=64')
        if dilation > 1:
            raise NotImplementedError("Dilation > 1 not supported in BasicBlock")
        self.bn1 = nn.BatchNorm2d(inplanes, eps=1e-05, )
        self.conv1 = conv3x3(inplanes, planes)
        self.bn2 = nn.BatchNorm2d(planes, eps=1e-05, )
        self.prelu = nn.PReLU(planes)
        self.conv2 = conv3x3(planes, planes, stride)
        self.bn3 = nn.BatchNorm2d(planes, eps=1e-05, )
        self.downsample = downsample
        self.stride = stride

    def forard_impl(self, x):
        identity = x
        out = self.bn1(x)
        out = self.conv1(out)
        out = self.bn2(out)
        out = self.prelu(out)
        out = self.conv2(out)
        out = self.bn3(out)
        if self.downsample is not None:
            identity = self.downsample(x)
        out += identity
        return out

    def forward(self, x):
        if self.training and using_ckpt:
            return checkpoint(self.forard_imlp, x)
        else:
            return self.forard_impl(x)


class IResNet(nn.Module):
    fc_scale = 7 * 7

    def __init__(self,
                 block, layers, dropout=0, num_features=512, zero_init_residual=False,
                 groups=1, width_per_group=64, replace_stride_with_dilation=None, fp16=False):
        super(IResNet, self).__init__()
        self.extra_gflops = 0.0
        self.fp16 = fp16
        self.inplanes = 64
        self.dilation = 1
        if replace_stride_with_dilation is None:
            replace_stride_with_dilation = [False, False, False]
        if len(replace_stride_with_dilation) != 3:
            raise ValueError("replace_stride_with_dilation should be None "
                             "or a 3-element tuple, got {}".format(replace_stride_with_dilation))
        self.groups = groups
        self.base_width = width_per_group
        self.conv1 = nn.Conv2d(3, self.inplanes, kernel_size=3, stride=1, padding=1, bias=False)
        self.bn1 = nn.BatchNorm2d(self.inplanes, eps=1e-05)
        self.prelu = nn.PReLU(self.inplanes)
        self.layer1 = self._make_layer(block, 64, layers[0], stride=2)
        self.layer2 = self._make_layer(block,
                                       128,
                                       layers[1],
                                       stride=2,
                                       dilate=replace_stride_with_dilation[0])
        self.layer3 = self._make_layer(block,
                                       256,
                                       layers[2],
                                       stride=2,
                                       dilate=replace_stride_with_dilation[1])
        self.layer4 = self._make_layer(block,
                                       512,
                                       layers[3],
                                       stride=2,
                                       dilate=replace_stride_with_dilation[2])
        self.bn2 = nn.BatchNorm2d(512 * block.expansion, eps=1e-05, )
        self.dropout = nn.Dropout(p=dropout, inplace=True)
        self.fc = nn.Linear(512 * block.expansion * self.fc_scale, num_features)
        self.features = nn.BatchNorm1d(num_features, eps=1e-05)
        nn.init.constant_(self.features.weight, 1.0)
        self.features.weight.requires_grad = False

        for m in self.modules():
            if isinstance(m, nn.Conv2d):
                nn.init.normal_(m.weight, 0, 0.1)
            elif isinstance(m, (nn.BatchNorm2d, nn.GroupNorm)):
                nn.init.constant_(m.weight, 1)
                nn.init.constant_(m.bias, 0)

        if zero_init_residual:
            for m in self.modules():
                if isinstance(m, IBasicBlock):
                    nn.init.constant_(m.bn2.weight, 0)

    def _make_layer(self, block, planes, blocks, stride=1, dilate=False):
        downsample = None
        previous_dilation = self.dilation
        if dilate:
            self.dilation *= stride
            stride = 1
        if stride != 1 or self.inplanes != planes * block.expansion:
            downsample = nn.Sequential(
                conv1x1(self.inplanes, planes * block.expansion, stride),
                nn.BatchNorm2d(planes * block.expansion, eps=1e-05, ),
            )
        layers = []
        layers.append(
            block(self.inplanes, planes, stride, downsample, self.groups,
                  self.base_width, previous_dilation))
        self.inplanes = planes * block.expansion
        for _ in range(1, blocks):
            layers.append(
                block(self.inplanes,
                      planes,
                      groups=self.groups,
                      base_width=self.base_width,
                      dilation=self.dilation))

        return nn.Sequential(*layers)

    def forward(self, x):
        with torch.amp.autocast('cuda', enabled=self.fp16):
            x = self.conv1(x)
            x = self.bn1(x)
            x = self.prelu(x)
            x = self.layer1(x)
            x = self.layer2(x)
            x = self.layer3(x)
            x = self.layer4(x)
            x = self.bn2(x)
            x = torch.flatten(x, 1)
            x = self.dropout(x)
        x = self.fc(x.float() if self.fp16 else x)
        x = self.features(x)
        return x


def _iresnet(arch, block, layers, pretrained, progress, **kwargs):
    model = IResNet(block, layers, **kwargs)
    if pretrained:
        raise ValueError()
    return model


def iresnet18(pretrained=False, progress=True, **kwargs):
    return _iresnet('iresnet18', IBasicBlock, [2, 2, 2, 2], pretrained,
                    progress, **kwargs)


def iresnet34(pretrained=False, progress=True, **kwargs):
    return _iresnet('iresnet34', IBasicBlock, [3, 4, 6, 3], pretrained,
                    progress, **kwargs)


def iresnet50(pretrained=False, progress=True, **kwargs):
    return _iresnet('iresnet50', IBasicBlock, [3, 4, 14, 3], pretrained,
                    progress, **kwargs)


def iresnet100(pretrained=False, progress=True, **kwargs):
    return _iresnet('iresnet100', IBasicBlock, [3, 13, 30, 3], pretrained,
                    progress, **kwargs)


def iresnet200(pretrained=False, progress=True, **kwargs):
    return _iresnet('iresnet200', IBasicBlock, [6, 26, 60, 6], pretrained,
                    progress, **kwargs)


def load_cosface(backbone: str):
    if backbone == "r18":
        model = iresnet18()
    elif backbone == "r50":
        model = iresnet50()
    elif backbone == "r100":
        model = iresnet100()
    return model

import os
import numpy
import torch
from torchvision import transforms


class FaceQuality:
    def __init__(self, ediffiqa_model, device='cuda'):
        if os.path.getsize(ediffiqa_model) < 100 * 1024 * 1024:
            model_type = 's'
        elif os.path.getsize(ediffiqa_model) < 200 * 1024 * 1024:
            model_type = 'm'
        else:
            model_type = 'l'
        self.device = torch.device(device)
        self.transform = transforms.Compose([
            transforms.ToTensor(),  # 将numpy图像(H x W x C) [0, 255] 转换为tensor (C x H x W) [0.0, 1.0]
            # transforms.Resize((112, 112)),  # 调整大小到112x112
            transforms.Normalize(mean=[0.5, 0.5, 0.5], std=[0.5, 0.5, 0.5])  # 归一化
        ])
        i_resnet = load_cosface({'s': "r18", 'm': "r50", 'l': "r100"}[model_type])
        # i_resnet.load_state_dict(torch.load(i_resnet_model, weights_only=True))
        # i_resnet.to(self.device)

        mlp = MLP()
        # mlp.to(self.device)
        self.model = eDifFIQA(i_resnet, mlp)
        self.model.load_state_dict(torch.load(ediffiqa_model, weights_only=True))
        self.model.to(self.device)
        self.model.eval()

    def predict(self, image):
        if isinstance(image, numpy.ndarray):
            image = self.transform(image).to(self.device).unsqueeze(0)
        return list(map(float, self.model(image)[1]))

    def classify(self, image):
        return self.predict(image)
